SystemOrganization addCategory: #'TimerDaemon-Test'!
SystemOrganization addCategory: #'TimerDaemon-Core'!

TestCase subclass: #TdTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TimerDaemon-Test'!

!TdTestCase methodsFor: 'testing' stamp: 'mu 2/28/2006 20:28'!
testBlockTimer
	"self debug: #testBlockTimer"
	| ord timer size |
	[ord _ OrderedCollection new.
	timer := TdBlockTimer named: 'test1' performBlock: [:t | ord add: #a] intervalSeconds: 1 .
	self
		should: [ord size = 0].
	timer priority: Processor userInterruptPriority.
	timer start.
	(Delay forMilliseconds: 500) wait.
	Processor yield.
	(Delay forMilliseconds: 500) wait.
	self
		should: [ord size >= 1].
	(Delay forMilliseconds: 500) wait.

	self should: [ord asSet size = 1].
	self should: [ord asSet asArray first = #a].

	timer := TdBlockTimer named: 'test1' performBlock: [:t | ord add: #b] intervalSeconds: 1 .
	timer priority: Processor userInterruptPriority.
	timer start.
	(Delay forMilliseconds: 500) wait.
	Processor yield.
	(Delay forMilliseconds: 500) wait.

	self should: [ord last = #b].
	timer stop.

	size _ ord size.
	(Delay forMilliseconds: 1000) wait.
	self should: [ord size = size].	
	
	]
		ensure: [timer
				ifNotNil: [timer release]]! !

!TdTestCase methodsFor: 'testing' stamp: 'mu 2/28/2006 20:14'!
testSimpleTimer
	"self debug: #testSimpleTimer"
	| timer |
	[timer := TdTestTimer default initialize.
	self
		should: [timer done = 0].
	timer priority: Processor userInterruptPriority.
	timer intervalSeconds: 1.
	timer start.
	(Delay forMilliseconds: 500) wait.
	self
		should: [timer done >= 1].
	(Delay forSeconds: 1) wait.
	self
		should: [timer done >= 2]]
		ensure: [timer
				ifNotNil: [timer release]]! !

ApplicationService subclass: #TdTimer
	instanceVariableNames: 'shouldRunAlways intervalSeconds'
	classVariableNames: 'WatchDog'
	poolDictionaries: ''
	category: 'TimerDaemon-Core'!

TdTimer class
	instanceVariableNames: 'default'!

TdTimer subclass: #TdBlockTimer
	instanceVariableNames: 'performBlock performMutex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TimerDaemon-Core'!

TdBlockTimer class
	instanceVariableNames: 'namedInstances'!

!TdBlockTimer class methodsFor: 'instance creation' stamp: 'mu 2/28/2006 19:51'!
basicNew
	^super basicNew initialize! !

!TdBlockTimer class methodsFor: 'class initialization' stamp: 'mu 2/26/2006 12:16'!
initialize
	super initialize.
	namedInstances _ nil! !

!TdBlockTimer class methodsFor: 'accessing' stamp: 'mu 2/28/2006 22:07'!
managedInstances
	self namedInstances ifNil: [^#()].
	^self namedInstances values! !

!TdBlockTimer class methodsFor: 'accessing' stamp: 'mu 2/26/2006 12:16'!
namedInstances
	namedInstances ifNil: [namedInstances _ IdentityDictionary new].
	^namedInstances! !

!TdBlockTimer class methodsFor: 'instance creation' stamp: 'mu 2/26/2006 11:54'!
named: aString performBlock: aBlock 
	^ self
		named: aString
		performBlock: aBlock
		intervalSeconds: self defaultIntervalSeconds! !

!TdBlockTimer class methodsFor: 'instance creation' stamp: 'mu 2/26/2006 11:53'!
named: aString performBlock: aBlock intervalSeconds: seconds 
	^ self
		named: aString
		performBlock: aBlock
		intervalSeconds: seconds
		priority: self defaultPriority! !

!TdBlockTimer class methodsFor: 'instance creation' stamp: 'mu 2/28/2006 20:09'!
named: aString performBlock: aBlock intervalSeconds: seconds priority: priority 
	| inst key |
	key := aString asSymbol.
	inst := self namedInstances
				at: key
				ifAbsent: [].
	inst
		ifNil: [inst := self namedInstances at: key put: (self basicNew name: aString;
							 performBlock: aBlock;
							 basicIntervalSeconds: seconds;
							 priority: priority)]
		ifNotNil: [inst stop; performBlock: aBlock; basicIntervalSeconds: seconds; priority: priority].
	^ inst! !

!TdBlockTimer class methodsFor: 'instance creation' stamp: 'mu 2/27/2006 23:07'!
performBlock: aBlock
	^self basicNew performBlock: aBlock! !

!TdBlockTimer class methodsFor: 'class initialization' stamp: 'mu 2/27/2006 23:13'!
release
	self releaseAllInstances! !

!TdBlockTimer class methodsFor: 'class initialization' stamp: 'mu 3/1/2006 00:03'!
releaseAllInstances
	"self releaseAllInstances"
	self namedInstances do: [:each | each release].
	namedInstances _ nil! !

!TdBlockTimer methodsFor: 'initialize-release' stamp: 'mu 2/28/2006 19:51'!
initialize
	super initialize.
	performMutex _ Semaphore forMutualExclusion! !

!TdBlockTimer methodsFor: 'actions' stamp: 'mu 2/28/2006 19:43'!
perform
	| return |
	performBlock ifNil: [^self].
	return _ performMutex critical: [performBlock value: self].
	^return! !

!TdBlockTimer methodsFor: 'accessing' stamp: 'mu 9/1/2005 12:05'!
performBlock
	^performBlock! !

!TdBlockTimer methodsFor: 'accessing' stamp: 'mu 2/28/2006 19:41'!
performBlock: aBlock 
	aBlock numArgs = 1
		ifFalse: [self release.
			self error: '''timer'' argument is expected'.
			^ self].
	performMutex
		critical: [performBlock := aBlock]! !

!TdBlockTimer methodsFor: 'registration' stamp: 'mu 3/11/2006 17:49'!
register
	"never registered as ApplicationService"! !

!TdBlockTimer methodsFor: 'initialize-release' stamp: 'mu 3/10/2006 20:38'!
release
	super release.
	performBlock _ nil.
	performMutex ifNotNilDo: [:m | [m isEmpty] whileFalse:[m signal]].
	performMutex _ nil.
	self unmanaged.
	! !

!TdBlockTimer methodsFor: 'accessing' stamp: 'mu 9/12/2005 08:25'!
shouldRun
	^performBlock notNil and: [super shouldRun] ! !

!TdBlockTimer methodsFor: 'initialize-release' stamp: 'mu 2/28/2006 23:28'!
unmanaged
	self class namedInstances
		removeKey: self name asSymbol
		ifAbsent: []! !

TdTimer subclass: #TdTestTimer
	instanceVariableNames: 'done'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TimerDaemon-Test'!

!TdTestTimer methodsFor: 'accessing' stamp: 'mu 2/26/2006 12:41'!
done
	^done! !

!TdTestTimer methodsFor: 'initialize-release' stamp: 'mu 2/26/2006 12:41'!
initialize
	super initialize.
	done _ 0.! !

!TdTestTimer methodsFor: 'actions' stamp: 'mu 2/26/2006 12:41'!
perform
	done _ done + 1! !

!TdTimer class methodsFor: 'accessing' stamp: 'mu 2/28/2006 23:57'!
allManagedInstances
	"TdTimer allManagedInstances"
	^TdTimer allSubclasses inject: Set new into: [:sum :each |
		sum addAll: each managedInstances; yourself
	].
! !

!TdTimer class methodsFor: 'instance creation' stamp: 'mu 8/22/2005 10:12'!
default
	default isNil
		ifTrue: [default := self basicNew initialize].
	^ default! !

!TdTimer class methodsFor: 'constants' stamp: 'mu 5/20/2005 19:35'!
defaultIntervalSeconds
	^ 10! !

!TdTimer class methodsFor: 'constants' stamp: 'mu 5/31/2005 00:48'!
defaultPriority
	^ Processor userSchedulingPriority - 1! !

!TdTimer class methodsFor: 'class initialization' stamp: 'mu 2/26/2006 11:49'!
flushAllSubInstances
	"self flushAllSubInstances."
	self allSubclassesDo: [:each | each flushInstances]! !

!TdTimer class methodsFor: 'class initialization' stamp: 'mu 2/26/2006 11:56'!
flushInstances
	"self flushInstances."
	default notNil
		ifTrue: [default class = self ifTrue: [default stop].
			default := nil].
	self allInstances
		do: [:each | 
			each stop.
			"each become: Object new"].
	Smalltalk garbageCollect.
	! !

!TdTimer class methodsFor: 'class initialization' stamp: 'mu 2/28/2006 23:14'!
initialize
	"self initialize"
	Smalltalk addToStartUpList: self.

	self release.
	self initWatchDog.

	! !

!TdTimer class methodsFor: 'watch dog' stamp: 'mu 3/1/2006 00:01'!
initWatchDog
	"self initWatchDog"
	self releaseWatchDog.
	WatchDog := TdBlockTimer
				named: self name , ':WatchDog'
				performBlock: [:timer |
					(TdTimer allManagedInstances select: [:each | each shouldRunAlways])
						do: [:each | each isRunning ifFalse: [each start]]]
				intervalSeconds: 20.
	WatchDog unmanaged.
	WatchDog priority: Processor userBackgroundPriority.
	WatchDog performThenStart! !

!TdTimer class methodsFor: 'accessing' stamp: 'mu 2/28/2006 20:41'!
managedInstances
	"default is just allInstances"
	^self allInstances! !

!TdTimer class methodsFor: 'instance creation' stamp: 'mu 5/20/2005 19:33'!
new
	self shouldNotImplement! !

!TdTimer class methodsFor: 'class initialization' stamp: 'mu 8/22/2005 10:56'!
release
	self releaseWatchDog.

	! !

!TdTimer class methodsFor: 'class initialization' stamp: 'MU 2/6/2006 17:11'!
releaseAllInstances
	self allInstancesDo: [:each | each release].! !

!TdTimer class methodsFor: 'watch dog' stamp: 'mu 3/10/2006 16:37'!
releaseWatchDog
	WatchDog notNil ifTrue: [WatchDog release].
	WatchDog _ nil! !

!TdTimer class methodsFor: 'snapshots' stamp: 'mu 2/28/2006 23:14'!
startUp
	self initWatchDog! !

!TdTimer methodsFor: 'accessing' stamp: 'mu 9/1/2005 12:36'!
basicIntervalSeconds: seconds 
	intervalSeconds := seconds.
	! !

!TdTimer methodsFor: 'accessing' stamp: 'mu 2/28/2006 10:04'!
interval
	"obsolete"
	^self intervalSeconds * 1000! !

!TdTimer methodsFor: 'accessing' stamp: 'mu 9/2/2005 17:16'!
intervalSeconds
	intervalSeconds
		ifNil: [intervalSeconds := self class defaultIntervalSeconds].
	^ intervalSeconds value! !

!TdTimer methodsFor: 'accessing' stamp: 'mu 3/7/2006 21:23'!
intervalSeconds: seconds 
	self basicIntervalSeconds: seconds.
	! !

!TdTimer methodsFor: 'accessing' stamp: 'mu 5/20/2005 19:33'!
interval: msec 
	"obsolete"
	self intervalSeconds: (msec / 1000) asFloat! !

!TdTimer methodsFor: 'actions' stamp: 'mu 2/26/2006 12:33'!
perform
	"override to perform something"! !

!TdTimer methodsFor: 'running' stamp: 'mu 2/28/2006 23:11'!
performThenStart
	self safePerform.
	self start! !

!TdTimer methodsFor: 'initialize-release' stamp: 'mu 3/10/2006 20:37'!
release
	shouldRunAlways := false.
	self unregister.
	! !

!TdTimer methodsFor: 'running' stamp: 'mu 2/28/2006 10:17'!
runWhile: aBlock 
	[aBlock value]
		whileTrue: [self sleepFor: self interval.
			self safePerform]! !

!TdTimer methodsFor: 'private' stamp: 'mu 2/28/2006 19:38'!
safePerform
	[self perform]
		on: Error
		do: [:ex | 
			self stop.
			ex pass]! !

!TdTimer methodsFor: 'accessing' stamp: 'mu 10/3/2005 18:53'!
shouldRunAlways
	shouldRunAlways ifNil: [shouldRunAlways := false].
	^shouldRunAlways! !

!TdTimer methodsFor: 'accessing' stamp: 'mu 10/3/2005 18:53'!
shouldRunAlways: aBoolean
	shouldRunAlways := aBoolean! !

!TdTimer methodsFor: 'running' stamp: 'mu 5/20/2005 19:33'!
startAfter: seconds 
	[(Delay forSeconds: seconds) wait.
	self start]
		forkAt: Processor userBackgroundPriority! !
TdBlockTimer initialize!
TdTimer initialize!
